/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package org.apithefire.persistence.config;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.TreeMap;

import javax.persistence.EntityManagerFactory;
import javax.persistence.PersistenceException;
import javax.persistence.spi.PersistenceProvider;
import javax.persistence.spi.PersistenceUnitInfo;

import org.apithefire.sql.connect.ConnectionProvider;
import org.apithefire.util.lang.Builder;
import org.apithefire.util.lang.Objects;

/**
 * An {@link EntityManagerFactory} builder.
 * 
 * @author <a href="mailto:bachew@gmail.com">Chew Boon Aik</a>
 */
public class EntityManagerFactoryBuilder implements Builder {

	private PersistenceProvider persistenceProvider;
	private final List<Class<?>> managedClasses = new ArrayList<Class<?>>();
	private ConnectionProvider connectionProvider;
	private final Map<String, String> propertyMap = new TreeMap<String, String>();
	
	/**
	 * Constructs a new instance.
	 */
	public EntityManagerFactoryBuilder() {
	}

	/**
	 * Sets persistence provider used to create entity manager factory.
	 * 
	 * @return this builder.
	 */
	public EntityManagerFactoryBuilder setPersistenceProvider(
			PersistenceProvider persistenceProvider) {
		this.persistenceProvider = Objects.nonNull(persistenceProvider);
		return this;
	}

	/**
	 * Adds the specified managed class.
	 * 
	 * @return this builder.
	 */
	public EntityManagerFactoryBuilder addManagedClass(Class<?> managedClass) {
		managedClasses.add(Objects.nonNull(managedClass));
		return this;
	}

	/**
	 * Sets the specified connection provider used to obtain database
	 * connection.
	 * 
	 * @return this builder.
	 */
	// TODO: should we use DataSource instead?
	public EntityManagerFactoryBuilder setConnectionProvider(
			ConnectionProvider connectionProvider) {
		this.connectionProvider = Objects.nonNull(connectionProvider);
		return this;
	}

	/**
	 * Sets the specified persistence provider specific property.
	 * 
	 * @return this builder.
	 */
	public EntityManagerFactoryBuilder setProperty(String name, String value) {
		propertyMap.put(Objects.nonNull(name), Objects.nonNull(value));
		return this;
	}

	/**
	 * Builds a new {@link EntityManagerFactory}.
	 * 
	 * @return the built {@link EntityManagerFactory}.
	 */
	public EntityManagerFactory build() {
		if (persistenceProvider == null) {
			throw new PersistenceException(
					"Persistence provider is not set.");
		}
		if (connectionProvider == null) {
			throw new PersistenceException("Connection provider is not set.");
		}
		
		String providerClassName = persistenceProvider.getClass().getSimpleName();
		List<String> managedClassNames = getManagedClassNames();
		Properties properties = getProperties();
		PersistenceUnitInfo info = new PersistenceUnitInfoImpl(
				providerClassName, managedClassNames, properties);
		Map<String, String> extraPropertyMap = Collections.emptyMap();
		
		return persistenceProvider.createContainerEntityManagerFactory(info,
				extraPropertyMap);
	}
	
	private List<String> getManagedClassNames() {
		if (managedClasses.isEmpty()) {
			throw new PersistenceException("Managed classes is empty.");
		}
		
		List<String> classNames = new ArrayList<String>();
		
		for (Class<?> managedClass : managedClasses) {
			classNames.add(managedClass.getName());
		}
		return classNames;
	}
	
	private Properties getProperties() {
		Properties properties = new Properties();
		
		if (!propertyMap.isEmpty()) {
			for (Map.Entry<String, String> property : propertyMap.entrySet()) {
				String key = property.getKey();
				String value = property.getValue();

				properties.put(key, value);
			}
		}
		return properties;
	}
	
}
